@@ -20,10 +20,10 @@ use winit::event::{
     ElementState, Modifiers, MouseButton, MouseScrollDelta, Touch as TouchEvent, TouchPhase,
 };
 #[cfg(target_os = "macos")]
-use winit::event_loop::EventLoopWindowTarget;
+use winit::event_loop::ActiveEventLoop;
 use winit::keyboard::ModifiersState;
 #[cfg(target_os = "macos")]
-use winit::platform::macos::EventLoopWindowTargetExtMacOS;
+use winit::platform::macos::ActiveEventLoopExtMacOS;
 use winit::window::CursorIcon;
 
 use alacritty_terminal::event::EventListener;
@@ -36,6 +36,8 @@ use alacritty_terminal::vi_mode::ViMotion;
 use alacritty_terminal::vte::ansi::{ClearMode, Handler};
 
 use crate::clipboard::Clipboard;
+#[cfg(target_os = "macos")]
+use crate::config::window::Decorations;
 use crate::config::{Action, BindingMode, MouseAction, SearchAction, UiConfig, ViAction};
 use crate::display::hint::HintMatch;
 use crate::display::window::Window;
@@ -105,7 +107,7 @@ pub trait ActionContext<T: EventListener> {
     fn message(&self) -> Option<&Message>;
     fn config(&self) -> &UiConfig;
     #[cfg(target_os = "macos")]
-    fn event_loop(&self) -> &EventLoopWindowTarget<Event>;
+    fn event_loop(&self) -> &ActiveEventLoop;
     fn mouse_mode(&self) -> bool;
     fn clipboard_mut(&mut self) -> &mut Clipboard;
     fn scheduler_mut(&mut self) -> &mut Scheduler;
@@ -125,6 +127,7 @@ pub trait ActionContext<T: EventListener> {
     fn inline_search_state(&mut self) -> &mut InlineSearchState;
     fn start_inline_search(&mut self, _direction: Direction, _stop_short: bool) {}
     fn inline_search_next(&mut self) {}
+    fn inline_search_input(&mut self, _text: &str) {}
     fn inline_search_previous(&mut self) {}
     fn hint_input(&mut self, _character: char) {}
     fn trigger_hint(&mut self, _hint: &HintMatch) {}
@@ -321,7 +324,10 @@ impl<T: EventListener> Execute<T> for Action {
             #[cfg(not(target_os = "macos"))]
             Action::Hide => ctx.window().set_visible(false),
             Action::Minimize => ctx.window().set_minimized(true),
-            Action::Quit => ctx.terminal_mut().exit(),
+            Action::Quit => {
+                ctx.window().hold = false;
+                ctx.terminal_mut().exit();
+            },
             Action::IncreaseFontSize => ctx.change_font_size(FONT_SIZE_STEP),
             Action::DecreaseFontSize => ctx.change_font_size(-FONT_SIZE_STEP),
             Action::ResetFontSize => ctx.reset_font_size(),
@@ -385,8 +391,11 @@ impl<T: EventListener> Execute<T> for Action {
             Action::CreateNewWindow => ctx.create_new_window(None),
             #[cfg(target_os = "macos")]
             Action::CreateNewTab => {
-                let tabbing_id = Some(ctx.window().tabbing_id());
-                ctx.create_new_window(tabbing_id);
+                // Tabs on macOS are not possible without decorations.
+                if ctx.config().window.decorations != Decorations::None {
+                    let tabbing_id = Some(ctx.window().tabbing_id());
+                    ctx.create_new_window(tabbing_id);
+                }
             },
             #[cfg(target_os = "macos")]
             Action::SelectNextTab => ctx.window().select_next_tab(),
@@ -631,6 +640,7 @@ impl<T: EventListener, A: ActionContext<T>> Processor<T, A> {
     /// Handle left click selection and vi mode cursor movement.
     fn on_left_click(&mut self, point: Point) {
         let side = self.ctx.mouse().cell_side;
+        let control = self.ctx.modifiers().state().control_key();
 
         match self.ctx.mouse().click_state {
             ClickState::Click => {
@@ -640,21 +650,21 @@ impl<T: EventListener, A: ActionContext<T>> Processor<T, A> {
                 self.ctx.clear_selection();
 
                 // Start new empty selection.
-                if self.ctx.modifiers().state().control_key() {
+                if control {
                     self.ctx.start_selection(SelectionType::Block, point, side);
                 } else {
                     self.ctx.start_selection(SelectionType::Simple, point, side);
                 }
             },
-            ClickState::DoubleClick => {
+            ClickState::DoubleClick if !control => {
                 self.ctx.mouse_mut().block_hint_launcher = true;
                 self.ctx.start_selection(SelectionType::Semantic, point, side);
             },
-            ClickState::TripleClick => {
+            ClickState::TripleClick if !control => {
                 self.ctx.mouse_mut().block_hint_launcher = true;
                 self.ctx.start_selection(SelectionType::Lines, point, side);
             },
-            ClickState::None => (),
+            _ => (),
         };
 
         // Move vi mode cursor to mouse click position.
@@ -804,7 +814,7 @@ impl<T: EventListener, A: ActionContext<T>> Processor<T, A> {
         if self.ctx.terminal().mode().contains(TermMode::FOCUS_IN_OUT) {
             let chr = if is_focused { "I" } else { "O" };
 
-            let msg = format!("\x1b[{}", chr);
+            let msg = format!("\x1b[{chr}");
             self.ctx.write_to_pty(msg.into_bytes());
         }
     }
@@ -999,17 +1009,18 @@ impl<T: EventListener, A: ActionContext<T>> Processor<T, A> {
         let mouse_bindings = self.ctx.config().mouse_bindings().to_owned();
 
         // If mouse mode is active, also look for bindings without shift.
-        let mut check_fallback = mouse_mode && mods.contains(ModifiersState::SHIFT);
+        let fallback_allowed = mouse_mode && mods.contains(ModifiersState::SHIFT);
+        let mut exact_match_found = false;
 
         for binding in &mouse_bindings {
             // Don't trigger normal bindings in mouse mode unless Shift is pressed.
-            if binding.is_triggered_by(mode, mods, &button) && (check_fallback || !mouse_mode) {
+            if binding.is_triggered_by(mode, mods, &button) && (fallback_allowed || !mouse_mode) {
                 binding.action.execute(&mut self.ctx);
-                check_fallback = false;
+                exact_match_found = true;
             }
         }
 
-        if check_fallback {
+        if fallback_allowed && !exact_match_found {
             let fallback_mods = mods & !ModifiersState::SHIFT;
             for binding in &mouse_bindings {
                 if binding.is_triggered_by(mode, fallback_mods, &button) {
@@ -1129,7 +1140,7 @@ mod tests {
         inline_search_state: &'a mut InlineSearchState,
     }
 
-    impl<'a, T: EventListener> super::ActionContext<T> for ActionContext<'a, T> {
+    impl<T: EventListener> super::ActionContext<T> for ActionContext<'_, T> {
         fn search_next(
             &mut self,
             _origin: Point,
@@ -1219,7 +1230,7 @@ mod tests {
         }
 
         #[cfg(target_os = "macos")]
-        fn event_loop(&self) -> &EventLoopWindowTarget<Event> {
+        fn event_loop(&self) -> &ActiveEventLoop {
             unimplemented!();
         }
 
@@ -1321,9 +1332,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Left,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id: DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id: WindowId::dummy(),
         },
         end_state: ClickState::Click,
         input_delay: Duration::ZERO,
@@ -1337,9 +1348,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Right,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id: DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id: WindowId::dummy(),
         },
         end_state: ClickState::Click,
         input_delay: Duration::ZERO,
@@ -1353,9 +1364,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Middle,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id: DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id: WindowId::dummy(),
         },
         end_state: ClickState::Click,
         input_delay: Duration::ZERO,
@@ -1369,9 +1380,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Left,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id: DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id: WindowId::dummy(),
         },
         end_state: ClickState::DoubleClick,
         input_delay: Duration::ZERO,
@@ -1385,9 +1396,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Left,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id: DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id: WindowId::dummy(),
         },
         end_state: ClickState::Click,
         input_delay: CLICK_THRESHOLD,
@@ -1401,9 +1412,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Left,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id:  DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id:  WindowId::dummy(),
         },
         end_state: ClickState::TripleClick,
         input_delay: Duration::ZERO,
@@ -1417,9 +1428,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Left,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id: DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id: WindowId::dummy(),
         },
         end_state: ClickState::Click,
         input_delay: CLICK_THRESHOLD,
@@ -1433,9 +1444,9 @@ mod tests {
             event: WindowEvent::MouseInput {
                 state: ElementState::Pressed,
                 button: MouseButton::Right,
-                device_id: unsafe { DeviceId::dummy() },
+                device_id: DeviceId::dummy(),
             },
-            window_id: unsafe { WindowId::dummy() },
+            window_id: WindowId::dummy(),
         },
         end_state: ClickState::Click,
         input_delay: Duration::ZERO,
