@@ -19,22 +19,24 @@ use crate::event::{Event, EventType};
 const ALACRITTY_SOCKET_ENV: &str = "ALACRITTY_SOCKET";
 
 /// Create an IPC socket.
-pub fn spawn_ipc_socket(options: &Options, event_proxy: EventLoopProxy<Event>) -> Option<PathBuf> {
-    // Create the IPC socket and export its path as env variable if necessary.
+pub fn spawn_ipc_socket(
+    options: &Options,
+    event_proxy: EventLoopProxy<Event>,
+) -> IoResult<PathBuf> {
+    // Create the IPC socket and export its path as env.
+
     let socket_path = options.socket.clone().unwrap_or_else(|| {
         let mut path = socket_dir();
         path.push(format!("{}-{}.sock", socket_prefix(), process::id()));
         path
     });
-    env::set_var(ALACRITTY_SOCKET_ENV, socket_path.as_os_str());
 
-    let listener = match UnixListener::bind(&socket_path) {
-        Ok(listener) => listener,
-        Err(err) => {
-            warn!("Unable to create socket: {:?}", err);
-            return None;
-        },
-    };
+    let listener = UnixListener::bind(&socket_path)?;
+
+    env::set_var(ALACRITTY_SOCKET_ENV, socket_path.as_os_str());
+    if options.daemon {
+        println!("ALACRITTY_SOCKET={}; export ALACRITTY_SOCKET", socket_path.display());
+    }
 
     // Spawn a thread to listen on the IPC socket.
     thread::spawn_named("socket listener", move || {
@@ -75,7 +77,7 @@ pub fn spawn_ipc_socket(options: &Options, event_proxy: EventLoopProxy<Event>) -
         }
     });
 
-    Some(socket_path)
+    Ok(socket_path)
 }
 
 /// Send a message to the active Alacritty socket.
@@ -111,7 +113,7 @@ fn find_socket(socket_path: Option<PathBuf>) -> IoResult<UnixStream> {
     if let Some(socket_path) = socket_path {
         // Ensure we inform the user about an invalid path.
         return UnixStream::connect(&socket_path).map_err(|err| {
-            let message = format!("invalid socket path {:?}", socket_path);
+            let message = format!("invalid socket path {socket_path:?}");
             IoError::new(err.kind(), message)
         });
     }
